'''
ASL constant definitions

THIS FILE IS GENERATED, DON'T UPDATE MANUALLY
'''

ASL_LEVEL_EMERG = 0
ASL_LEVEL_ALERT = 1
ASL_LEVEL_CRIT = 2
ASL_LEVEL_ERR = 3
ASL_LEVEL_WARNING = 4
ASL_LEVEL_NOTICE = 5
ASL_LEVEL_INFO = 6
ASL_LEVEL_DEBUG = 7
ASL_STRING_EMERG = 'Emergency'
ASL_STRING_ALERT = 'Alert'
ASL_STRING_CRIT = 'Critical'
ASL_STRING_ERR = 'Error'
ASL_STRING_WARNING = 'Warning'
ASL_STRING_NOTICE = 'Notice'
ASL_STRING_INFO = 'Info'
ASL_STRING_DEBUG = 'Debug'
ASL_QUERY_OP_CASEFOLD = 16
ASL_QUERY_OP_PREFIX = 32
ASL_QUERY_OP_SUFFIX = 64
ASL_QUERY_OP_SUBSTRING = 96
ASL_QUERY_OP_NUMERIC = 128
ASL_QUERY_OP_REGEX = 256
ASL_QUERY_OP_EQUAL = 1
ASL_QUERY_OP_GREATER = 2
ASL_QUERY_OP_GREATER_EQUAL = 3
ASL_QUERY_OP_LESS = 4
ASL_QUERY_OP_LESS_EQUAL = 5
ASL_QUERY_OP_NOT_EQUAL = 6
ASL_QUERY_OP_TRUE = 7
ASL_KEY_TIME = 'Time'
ASL_KEY_TIME_NSEC = 'TimeNanoSec'
ASL_KEY_HOST = 'Host'
ASL_KEY_SENDER = 'Sender'
ASL_KEY_FACILITY = 'Facility'
ASL_KEY_PID = 'PID'
ASL_KEY_UID = 'UID'
ASL_KEY_GID = 'GID'
ASL_KEY_LEVEL = 'Level'
ASL_KEY_MSG = 'Message'
ASL_KEY_READ_UID = 'ReadUID'
ASL_KEY_READ_GID = 'ReadGID'
ASL_KEY_EXPIRE_TIME = 'ASLExpireTime'
ASL_KEY_MSG_ID = 'ASLMessageID'
ASL_KEY_SESSION = 'Session'
ASL_KEY_REF_PID = 'RefPID'
ASL_KEY_REF_PROC = 'RefProc'
ASL_KEY_AUX_TITLE = 'ASLAuxTitle'
ASL_KEY_AUX_UTI = 'ASLAuxUTI'
ASL_KEY_AUX_URL = 'ASLAuxURL'
ASL_KEY_AUX_DATA = 'ASLAuxData'
ASL_KEY_OPTION = 'ASLOption'
ASL_KEY_SENDER_INSTANCE = 'SenderInstance'
ASL_TYPE_MSG = 0
ASL_TYPE_QUERY = 1
ASL_FILTER_MASK_EMERG = 1
ASL_FILTER_MASK_ALERT = 2
ASL_FILTER_MASK_CRIT = 4
ASL_FILTER_MASK_ERR = 8
ASL_FILTER_MASK_WARNING = 16
ASL_FILTER_MASK_NOTICE = 32
ASL_FILTER_MASK_INFO = 64
ASL_FILTER_MASK_DEBUG = 128
ASL_OPT_STDERR = 1
ASL_OPT_NO_DELAY = 2
ASL_OPT_NO_REMOTE = 4
ASL_LOG_DESCRIPTOR_READ = 1
ASL_LOG_DESCRIPTOR_WRITE = 2



"""
Bindings to the Apple System Log facility.
"""
from __future__ import absolute_import

__version__ = "1.0"

from ._asl import *
from ._constants import *

asl_open = aslclient
asl_new = aslmsg


def ASL_FILTER_MASK(level):
    return 1 << level


def ASL_FILTER_MASK_UPTO(level):
    return (1 << (level + 1)) - 1

LEVEL2STRING = {
    ASL_LEVEL_EMERG: ASL_STRING_EMERG,
    ASL_LEVEL_ALERT: ASL_STRING_ALERT,
    ASL_LEVEL_CRIT: ASL_STRING_CRIT,
    ASL_LEVEL_ERR: ASL_STRING_ERR,
    ASL_LEVEL_WARNING: ASL_STRING_WARNING,
    ASL_LEVEL_NOTICE: ASL_STRING_NOTICE,
    ASL_LEVEL_INFO: ASL_STRING_INFO,
    ASL_LEVEL_DEBUG: ASL_STRING_DEBUG,
}

STRING2LEVEL = {
    ASL_STRING_EMERG: ASL_LEVEL_EMERG,
    ASL_STRING_ALERT: ASL_LEVEL_ALERT,
    ASL_STRING_CRIT: ASL_LEVEL_CRIT,
    ASL_STRING_ERR: ASL_LEVEL_ERR,
    ASL_STRING_WARNING: ASL_LEVEL_WARNING,
    ASL_STRING_NOTICE: ASL_LEVEL_NOTICE,
    ASL_STRING_INFO: ASL_LEVEL_INFO,
    ASL_STRING_DEBUG: ASL_LEVEL_DEBUG,
}



from __future__ import absolute_import, print_function
import argparse
import os
import sys
import collections

if sys.version_info[0] == 2:
    # Use string.Formatter on Python 2 because str doesn't
    # have a 'format_map' method yet.
    from string import Formatter

import asl


LEVELS = list(sorted(asl.STRING2LEVEL, key=lambda l: asl.STRING2LEVEL[l]))

parser = argparse.ArgumentParser(description="ASL command-line interface", prog=__package__)
parser.add_argument('--version', action='version', version='%(prog)s ' + asl.__version__)

sub = parser.add_subparsers(help="Sub-commands")

parser_consolelog = sub.add_parser('consolelog', help='Log to Console.app')
parser_consolelog.set_defaults(action='consolelog')
parser_consolelog.add_argument(
    '-i', '--ident',
    action='store', metavar='IDENT',
    default=os.getlogin(), help='Source identifier (default: %(default)s)'
)
parser_consolelog.add_argument(
    '-l', '--level',
    action='store', metavar='LEVEL',
    default=asl.ASL_STRING_NOTICE, choices=LEVELS,
    help='Logging level (default: %(default)s, valid values: %(choices)s)'
)
parser_consolelog.add_argument('message', help='The message', metavar='MESSAGE', nargs='+')

parser_sendlog = sub.add_parser('sendlog', help='Write log message')
parser_sendlog.set_defaults(action='sendlog')
parser_sendlog.add_argument(
    '-k', nargs=2, action='append',
    dest='keys', metavar=('KEY', 'VALUE'),
    help='Add a key and value to the message')

parser_query = sub.add_parser('query', help='Perform ASL query')
parser_query.set_defaults(action='query')
parser_query.add_argument(
    '-f', '--format',
    action='store', dest='format', default=None,
    metavar='FMT', help='Format string for showing the record')
parser_query.add_argument(
    '-C', action='store_true',
    dest='show_console', help='include Console.app log entries')
parser_query.add_argument(
    '-k', action='append', dest='query',
    nargs=3, metavar=('KEY', 'OP', 'VALUE'),
    help='add query element')
parser_query.add_argument(
    '-e', action='append', dest='exists',
    metavar='KEY', help='query that key exists')


def do_consolelog(options):
    ident = options.ident
    message = ' '.join(options.message)
    level = options.level

    cli = asl.aslclient(ident=ident, facility='com.apple.console', options=0)
    msg = asl.aslmsg(asl.ASL_TYPE_MSG)
    msg[asl.ASL_KEY_FACILITY] = "com.apple.console"
    msg[asl.ASL_KEY_LEVEL] = level
    msg[asl.ASL_KEY_READ_UID] = str(os.getuid())

    cli.log(msg, asl.STRING2LEVEL[level], message)


def do_sendlog(opts):
    if not opts.keys:
        print("No message arguments specified", file=sys.stderr)
        sys.exit(1)

    cli = asl.aslclient(ident=None, facility='user', options=0)
    msg = asl.aslmsg(asl.ASL_TYPE_MSG)
    for k, v in opts.keys:
        msg[k] = v

    cli.send(msg)


OP_MAP = {
    'eq': asl.ASL_QUERY_OP_EQUAL,
    'ne': asl.ASL_QUERY_OP_NOT_EQUAL,
    'gt': asl.ASL_QUERY_OP_GREATER,
    'ge': asl.ASL_QUERY_OP_GREATER_EQUAL,
    'le': asl.ASL_QUERY_OP_LESS_EQUAL,
    'lt': asl.ASL_QUERY_OP_LESS,
    'match': asl.ASL_QUERY_OP_REGEX,
    'Ceq': asl.ASL_QUERY_OP_EQUAL | asl.ASL_QUERY_OP_CASEFOLD,
    'Cne': asl.ASL_QUERY_OP_NOT_EQUAL | asl.ASL_QUERY_OP_CASEFOLD,
    'Cgt': asl.ASL_QUERY_OP_GREATER | asl.ASL_QUERY_OP_CASEFOLD,
    'Cge': asl.ASL_QUERY_OP_GREATER_EQUAL | asl.ASL_QUERY_OP_CASEFOLD,
    'Cle': asl.ASL_QUERY_OP_LESS_EQUAL | asl.ASL_QUERY_OP_CASEFOLD,
    'Clt': asl.ASL_QUERY_OP_LESS | asl.ASL_QUERY_OP_CASEFOLD,
    'Cmatch': asl.ASL_QUERY_OP_REGEX | asl.ASL_QUERY_OP_CASEFOLD,
    '==': asl.ASL_QUERY_OP_EQUAL | asl.ASL_QUERY_OP_NUMERIC,
    '!=': asl.ASL_QUERY_OP_NOT_EQUAL | asl.ASL_QUERY_OP_NUMERIC,
    '>': asl.ASL_QUERY_OP_GREATER | asl.ASL_QUERY_OP_NUMERIC,
    '>=': asl.ASL_QUERY_OP_GREATER_EQUAL | asl.ASL_QUERY_OP_NUMERIC,
    '<=': asl.ASL_QUERY_OP_LESS_EQUAL | asl.ASL_QUERY_OP_NUMERIC,
    '<': asl.ASL_QUERY_OP_LESS | asl.ASL_QUERY_OP_NUMERIC,
    'startswith': asl.ASL_QUERY_OP_EQUAL | asl.ASL_QUERY_OP_PREFIX,
    'endswith': asl.ASL_QUERY_OP_EQUAL | asl.ASL_QUERY_OP_SUFFIX,
    'contains': asl.ASL_QUERY_OP_EQUAL | asl.ASL_QUERY_OP_SUBSTRING,
    'Cstartswith': asl.ASL_QUERY_OP_EQUAL | asl.ASL_QUERY_OP_PREFIX | asl.ASL_QUERY_OP_CASEFOLD,
    'Cendswith': asl.ASL_QUERY_OP_EQUAL | asl.ASL_QUERY_OP_SUFFIX | asl.ASL_QUERY_OP_CASEFOLD,
    'Ccontains': asl.ASL_QUERY_OP_EQUAL | asl.ASL_QUERY_OP_SUBSTRING | asl.ASL_QUERY_OP_CASEFOLD,
}

def do_query(opts):
    cli = asl.aslclient(ident=None, facility='user', options=0)
    msg = asl.aslmsg(asl.ASL_TYPE_QUERY)
    if opts.show_console:
        msg.set_query(asl.ASL_KEY_FACILITY, "com.apple.console", asl.ASL_QUERY_OP_EQUAL)

    # Other search parameters
    if opts.query:
        valid = True
        for key, op, value in opts.query:
            try:
                msg.set_query(key, value, OP_MAP[op])

            except KeyError:
                valid = False
                print("Invalid query operation:", op, file=sys.stderr)

        if not valid:
            sys.exit(1)

    if opts.exists:
        for key in opts.exists:
            msg.set_query(key, '', asl.ASL_QUERY_OP_TRUE)

    for record in cli.search(msg):
        if opts.format is None:
            for key in sorted(record.keys()):
                print("{} {}".format(key, record[key]))
            print()

        else:
            fmt_arg = collections.defaultdict(str)
            fmt_arg.update({k: record[k] for k in record.keys()})

            if sys.version_info[0] == 2:
                print(Formatter().vformat(opts.format, (), fmt_arg))
            else:
                print(opts.format.format_map(fmt_arg))


def main():
    opts = parser.parse_args()
    if opts.action == 'consolelog':
        do_consolelog(opts)

    elif opts.action == 'sendlog':
        do_sendlog(opts)

    elif opts.action == 'query':
        do_query(opts)

    else:
        raise NotImplementedError("Action: %s" % (opts.action,))

if __name__ == "__main__":
    main()
